<?php

/**
 * The Replace class handles the replacement
 * of Strings that can be defined in the contetnt.xml
 * file. Specific functions provide replacement for
 * specific HTML tags.
 *  
 * $Id: Replace.class.php,v 1.1.1.1 2006/09/17 15:41:38 Andi Exp $
 *
 * @license    http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright  Copyright in 2006 by Andi Hotz
 * @copyright  GNU Lesser General Public License
 * @author     Andi Hotz
 * @version    $Revision: 1.1.1.1 $
 * @package    ODFLib
 * @since      0.0.1
 */
class Replace{

  /**
   * namespace OpenDocument text
   */
	const TEXT = 'urn:oasis:names:tc:opendocument:xmlns:text:1.0';  
        
	/**
	 * DomDocument representation of an XML file
	 *
	 * @var DOMDocument
	 */
	private $dom ;
	
	/**
	 * Current node in den DOMDocument
	 */
	private $currentNode;
	
	/**
	 * Constructor initialises DOM structure with
	 * $xmlFileContents. The current node is initialised
	 * with the body tag.
	 * @param string $xmlFileContets XML file in a string representation.
	 */
	public function __construct($xmlFileContets){
		$this->dom = new DOMDocument;
		$this->dom->loadXML($xmlFileContets);
		$domList=$this->dom->getElementsByTagName('body');
		$this->currentNode=new DOMNode();
		$this->currentNode=$domList->item(0);
	}
	
	/**
	 * Walk through all paragraphs ('p' tag) of
	 * the DOMDocument untill the first is found
	 * that first child contains the replace string.
	 * The currentNode pointer is set to the parent node
	 * of the text node.
	 * @param string $replace
	 * @throws template string could not be found
	 */
	private function findString($replace){
	    require_once('odfNodes/Paragraph.class.php');
		$tags = $this->dom->getElementsByTagNameNS(self::TEXT ,'p');
		$i=0;
		while ($field = $tags->item($i)){
			if ($field->firstChild && $field->nodeValue == $replace){
				$this->currentNode=$field;
				return;
			} else {
				$i++;
			}
		}
		throw new Exception("The template string ".$replace." could not be found.");
	}
	
	/**
	 * The replace string is thought in the the document
	 * by use of methode findString(string). The replace string
	 * is replaced by the replacement string
	 *
	 * @param string $replace should be replaced
	 * @param string $replacement replaces $replace
	 */
	public function replaceText($replace,$replacement){
		$this->findString($replace);
		$textNode = $this->dom->createTextNode($replacement);
		$this->currentNode->replaceChild($textNode,$this->currentNode->firstChild);
	}
	
	/**
	 * Add a new paragraph at the position of currentNode
	 * @param  string $paragraphText text of the new paragraph
	 */
	public function addParagraph($paragraphText){
		require_once('odfNodes/Paragraph.class.php');
		$paragraph = new Paragraph($this->dom);
		$paragraph->addParagraph($paragraphText,$this->currentNode);
		$this->procedeNext();
	}
	/**
	 * Add a new paragraph at the position of currentNode
	 * @param  string $paragraphText text of the new paragraph
	 * @param string $style style of the paragraph
	 */
	public function addStyledParagraph($paragraphText,$style){
		require_once('odfNodes/Paragraph.class.php');
		$paragraph = new Paragraph($this->dom);
		$paragraph->addStyledParagraph($paragraphText,$this->currentNode,$style);
		$this->procedeNext();
	}
	
	/**
	 * Remove the Text replace. The tag is not removed.
	 * @param string $replace Template-String
	 */
	public function removeReplaceString($replace){
		$this->findString($replace);
		$node = $this->currentNode;
		$node->removeChild($this->currentNode->firstChild);
	}
	
	/**
	 * Remove the previous sibling of the current node
	 */
	public function removePreviousSibling(){
		$prev = $this->currentNode->previousSibling;
		$this->currentNode->removeChild($prev);
	}

	/**
	 * Add a new paragraph at the position of currentNode
	 * @param  string $paragraphText text of the new heading
	 * @param int $level of the Heading
	 */
	public function addHeadding($head,$level){
		require_once('odfNodes/Heading.class.php');
		$paragraph = new Heading($this->dom);
		$paragraph->addHeading($head,$level,$this->currentNode);
		$this->procedeNext();
	}
	/**
	 * Add a new paragraph at the position of currentNode
	 * @param  string $paragraphText text of the new heading
	 * @param int $level of the Heading
	 * @param string $style style of the paragraph
	 */
	public function addStyledHeading($paragraphText,$level,$style){
		require_once('odfNodes/Heading.class.php');
		$paragraph = new Heading($this->dom);
		$paragraph->addStyledHeading($paragraphText,$level,$this->currentNode,$style);
		$this->procedeNext();
	}
	
	/**
	 * After an new node is added the current node must be set onto the
	 * new, the next sibling.
	 *
	 */
	private function procedeNext(){
		$this->currentNode=$this->currentNode->nextSibling;
	}
	
	/**
	 * Insert an $newNode after an $oldNode in the
	 * $dom structure.
	 *
	 * @param DOMDocument $dom DOM Structure
	 * @param DOMNode $oldNode Node after witch should be inserted
	 * @param DOMNode $newNode Node to be inserted
	 * @return The node inserted
	 */
	public static function insertAfter(DOMDocument $dom, DOMNode $oldNode, DOMNode $newNode){
	    if ($oldNode->nextSibling != null){
		    // There are more children after the node after which should be inserted
		    return $oldNode->nextSibling->insertBefore($newNode,$oldNode->nextSibling);
		} else {
		    // The node after witch should be inserted is the last child
		    return $oldNode->parentNode->appendChild($newNode);
		}
	}
	
	public function addTable(array $tableData,$withHeader){
	    if (!$this->checkTableData($tableData)){
	        throw new Exception("The rows in tableData don't have the same number of columns");
	    }
	    require_once('odfNodes/Table.class.php');
	    require_once('odfNodes/TableRow.class.php');
	    require_once('odfNodes/TableCell.class.php');
	    $numOfCols = count($tableData[0]);
	    $numOfRows = count($tableData);
	    $table = new Table($this->dom,$numOfCols);
	    $tableNode = $table->addTable($this->currentNode);
	    $this->currentNode = $tableNode;
	    for ($i=0;$i<$numOfRows;$i++){
	        $tableRow = new TableRow($this->dom);
	        $row = $tableData[$i];
			$tableRowNode;
	        if ($i==0 && $withHeader){
	            $tableRowNode = $tableRow->addTableHeaderRow($tableNode);
	        } else {
	            $tableRowNode = $tableRow->addTableRow($tableNode);
	        }
	        $this->currentNode = $tableRowNode;
	        for ($j=0;$j<$numOfCols;$j++){
	            $cell = $row[$j];
	            $tableCell = new TableCell($this->dom);
	            $this->currentNode = $tableCell->addTableCell($tableRowNode,'string',$cell);
	            if ($j==$numOfCols-1){
	                // Last cell => step out
	                $this->currentNode=$this->currentNode->parentNode;
	            }
	        }
	        if ($j==$numOfRows-1){
                // Last row => step out
                $this->currentNode=$this->currentNode->parentNode;
            } else {
                $this->procedeNext();
            }
   	    }
	}
	
	/**
	 * Check if each row in $tableData has the
	 * same number of columns.
	 * @param array $tableData array containing the rows of a
	 * table as arrays
	 * @return true if the $tableData array is well formed, else
	 * false;
	 */
	private function checkTableData(array $tableData){
	    $numOfCols=count($tableData[0]);
	    foreach($tableData as $row){
	        if (count($row)!=$numOfCols){
	            print_r($row);
	           return false; 
	       }
	    }
	    return true;
	}
	
	/**
	 * Gets the name and value of the current node
	 *
	 * @return string with the node name and its value.
	 */
	private function getCurrentNode(){
	    return "nodeName: ".$this->currentNode->nodeName.", nodeValue: ".$this->currentNode->nodeValue;
	}
	
	/**
	 * Converts the DOM structure into a string
	 * that can be written to a file by the IO class
	 * @return string representation of the DOM structure.
	 */
	public function toString(){
	    return $this->dom->saveXML();
	}
	
}
?>